use crate::{Image, Tileset, Vec2};
use num::complex::ComplexFloat;
use std::rc::Rc;

//odd-q vertical layout https://www.redblobgames.com/grids/hexagons
pub struct HexMap {
    size: Vec2<i32>,
    data: Vec<i32>,
    tileset: Rc<Tileset>,
    pix_tile_off: Vec2<i32>,
}

impl HexMap {
    //pub static
    //pix_tile_off: x offset per tile, up/down alternating y offset on x axis
    pub fn new(size: Vec2<i32>, tileset: Rc<Tileset>, pix_tile_off: Vec2<i32>) -> Self {
        HexMap {
            size,
            data: vec![0; size.size()],
            tileset,
            pix_tile_off,
        }
    }

    //pub
    pub fn get(&self, coord: Vec2<i32>) -> i32 {
        self.data[self.coord_to_idx(coord)]
    }

    pub fn pixel_to_coord(&self, pixel: Vec2<i32>) -> Vec2<i32> {
        let tilesize = self.tileset.tile_size();
        let xrepeat = pixel.x % self.pix_tile_off.x;
        let mut x = pixel.x / self.pix_tile_off.x;
        let py = if x % 2 == 0 {
            pixel.y
        } else {
            pixel.y - self.pix_tile_off.y
        };
        let yrepeat = py % tilesize.y;
        let mut y = py / tilesize.y;
        let outside = self.tileset.get(0).get_pixel(Vec2 {
            x: xrepeat,
            y: yrepeat,
        }) == 0;
        if outside {
            x -= 1;
            if yrepeat > tilesize.y / 2 {
                y += 1;
            }
            y -= x % 2;
        }
        let slice = xrepeat / (tilesize.x / 4);
        Vec2 { x, y }
    }

    pub fn set(&mut self, coord: Vec2<i32>, val: i32) {
        let idx = self.coord_to_idx(coord);
        self.data[idx] = val;
    }

    pub fn size(&self) -> Vec2<i32> {
        self.size
    }

    //pubcrate
    pub(crate) fn draw_tile_to_image(
        &self,
        target: &mut Image,
        coord: Vec2<i32>,
        offset: Vec2<i32>,
    ) {
        let x = coord.x * self.pix_tile_off.x;
        let y = coord.y * self.tileset.tile_size().y + (coord.x % 2) * self.pix_tile_off.y;
        target.draw_image(
            Vec2 { x, y } + offset,
            self.tileset.get(self.data[self.coord_to_idx(coord)]),
        )
    }

    //priv
    fn coord_to_idx(&self, coord: Vec2<i32>) -> usize {
        (coord.x + coord.y * self.size.x) as usize
    }
}
